/* eslint-disable */
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http, Router,
} from '@zeedhi/core';
import {
  Row, Form, IComponentRender, CheckboxMultiple,
  Button, Select, SelectMultiple, TextInput, Time, AlertService, Switch, Date as DateInput, DialogService,
} from '@zeedhi/common';

export class CadastroController {
  public modalController = Loader.getInstance('NovoCriterioController');
  public carregandoModal = Loader.getInstance('ModalLoading');

  public dadosTarefa: IDictionary = {
    selectTipoLocal: [],
    selectNomeLocal: [],
    selectChecklist: [],
    selectFuncionario: [],
    selectTarefa: [],
  };

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public setTipoLocal() {
    Http.get('/getTipoLocal').then((response: IDictionary) => {
      response.data.dataset.tipoLocal.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectTipoLocal.push(this.setDados(dados.NMTIPOLOCALCLI, dados.NRTIPOLOCALCLI));
      });
    });
  }

  public setLocal() {
    Http.get('/getLocal').then((response: IDictionary) => {
      response.data.dataset.local.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectNomeLocal.push(this.setDados(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
      });
    });
  }

  public setTipoChecklist() {
    Http.get('/getChecklistTarefa').then((response: IDictionary) => {
      response.data.dataset.checklist.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectChecklist.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
      });
    });
  }

  public setItensChecklist({ component }: { component:Select }) {
    Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = true;
    const chkMult = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
    chkMult!.datasource!.data = [];
    chkMult!.isVisible = false;
    const selectTipoTarefa: IDictionary = [];
    Http.post('/getItensChecklist', { checklist: component.value }).then((response: IDictionary) => {
      response.data.dataset.tipoTarefa.forEach((dados: IDictionary) => {
        selectTipoTarefa.push(this.setDados(dados.NMATIVIDADE, dados.NRATIVIDADECHK));
      });
    });
    const selectItem = Metadata.getInstance<SelectMultiple>('selectTipoTarefa');
    const selectTarefa = Metadata.getInstance<SelectMultiple>('selectTarefa');
    selectItem.datasource.data = selectTipoTarefa as IDictionary[];
    selectItem.setValue([]);
    selectTarefa.datasource.data = [];
    selectTarefa.setValue([]);
  }

  public setTarefa({ component }: { component:Select }) {
    Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = true;
    const chkMult = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
    chkMult!.datasource!.data = [];
    chkMult!.isVisible = false;
    const selectTipoTarefa: IDictionary = [];
    Http.post('/getTarefa', { checklist: component.value }).then((response: IDictionary) => {
      response.data.dataset.tarefa.forEach((dados: IDictionary) => {
        selectTipoTarefa.push(this.setDados(dados.NMTAREFA, dados.NRTAREFACHK));
      });
    });
    const select = Metadata.getInstance<SelectMultiple>('selectTarefa');
    select.datasource.data = selectTipoTarefa as IDictionary[];
    select.setValue([]);
  }

  public getCriteriosDeAvaliacao() {  
    Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = false;
    const criterioList = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
    criterioList!.datasource!.data! = [];
    criterioList!.value = [];
    Http.post('/getCriterioAtividade', {
      NRCHECKLIST: Metadata.getInstance<SelectMultiple>('selectChecklist').value,
      NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('selectTipoTarefa').value,
      NRTAREFACHK: Metadata.getInstance<SelectMultiple>('selectTarefa').value,
    }).then((response: IComponentRender) => {
      response.data.dataset.criterioTarefa.forEach((dados: IDictionary) => {
        criterioList!.datasource!.data!.push({
          value: dados.DSCRITERIOTAREFA,
          id: dados.NRCRITERIOTAREFACHK,
        });
      });
      criterioList.isVisible = true;
    });
  }

  public setFuncionario() {
    Http.get('/getFuncionario').then((response: IDictionary) => {
      response.data.dataset.funcionario.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectFuncionario.push(this.setDados(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
      });
    });
  }

  public inicializarDados() {
    this.setLocal();
    this.setTipoLocal();
    this.setTipoChecklist();
    this.setFuncionario();
  }

  public getDadosInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosTarefa[(component.name.includes('selectFuncionario')) ? 'selectFuncionario' : component.name];
  }

  // Comandos do modal
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    this.modalController.openBasicModal();
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  public modelCrterio(criterio: IDictionary) {
    return {
      name: 'criterioTarefa',
      component: 'ZdCheckbox',
      label: criterio.name,
      cssClass: 'zd-mx-3',
      value: criterio.value,
    };
  }

  public novoCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chk = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
      const valor = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chk!.datasource!.data!.push({
        value: valor,
        id: valor,
      });
      this.fecharModal();
    }
  }

  public botaoCriarTarefa() {
    // Variavel onde as datas irão ficar salva no formato de string
    const datas: IDictionary = [];
    // Pegando os valores de data do campo ZdDate
    let data = Metadata.getInstance<DateInput>('selectDataDe').value;
    let data2 = Metadata.getInstance<DateInput>('selectDataAte').value;
    // Criando uma validação se a data esta preenchida com todos os digitos
    // Verificando se a data final e igual ou superior a inicial
    let campoDataValido = ((data != null) ?  data.length === 10 : false) && ((data2 != null) ?  data2.length === 10 : false);
    if (campoDataValido) {
      // Transformando a variável em array para sebapara dia, mês e ano
      data = data.split('/');
      data2 = data2.split('/');
      // Usando as array da data para criar um Object Date.
      data = new Date(`${data[1]}/${data[0]}/${data[2]}`);
      data2 = new Date(`${data2[1]}/${data2[0]}/${data2[2]}`);
      campoDataValido = campoDataValido && (data <= data2);
    }
    // Validando todos os campos forms da tela e criando a variável com o estado da validação
    const form = Metadata.getInstance<Form>('FormularioPrincipal').validate();
    const form1 = Metadata.getInstance<Form>('FormComHorarioEMedia').validate();
    const form2 = Metadata.getInstance<Form>('FormComFuncionariosDaTarefa').validate();
    // Instanciando uma variavel com o campo do checkbox de datas
    const dias = Metadata.getInstance<CheckboxMultiple>('checkboxMultipleDate').value;
    if (!campoDataValido) {
      this.msg('Insira um periodo valida','info')
    } else if (dias.length === 0) {
      this.msg('Selecione o(s) dia(s) da semana','info');
    } else {
      // loop para percorrer as datas do período
      for (let dataAux = data; data <= data2; dataAux.setDate(dataAux.getDate() + 1)) {
        // Validação se o dia da data e compativel com os "Dias da semana" marcados
        if (dias.includes(dataAux.getDay())) {
          // Se a data iterada for compativel com o dia da semana ele vai salvar essa data em uma arry no formato string
          datas.push(this.dataFormat(dataAux));
        }
      }
      /* Essa validação e para caso a pessoa tenha escolhido um periodo valido mas o dia escolhido não se encontra 
      nesse intervalo de datas */
      if (datas.length === 0) {
        // Exibe a msg com o erro
        this.msg('O periodo selecionado não possui o(s) dia(s) da semana escolhido.','warning');
      }
    }
    //Condicional para mandar os dados para o backend
    // Nessa condicional ele faz a validação de todos os forms estão preenchidos, e verifica se alguma data foi salva.
    if (form && form1 && form2 && datas.length > 0) {
      const selectObjectFuncionario =  Metadata.getInstance<Select>('selectFuncionario').value;
      const texto =  'Você esta criando uma tarefa sem vincular um funcionário, deseja continuar?';
      if (selectObjectFuncionario === ''  || selectObjectFuncionario === null){//Diego Sabino - 5/4/2021
        this.mensagem(texto, datas);
      } else {
        this.postTarefa(datas);
      }
    }
  }

  public criaNovoHorario(n: number) {
    return [
      {
        name: 'selectHoraInicio',
        component: 'ZdTime',
        autocomplete: false,
        cssClass: 'zd-px-2',
        label: 'Inicio',
        grid: {
          cols: 6,
        },
        validations: {
          required: {},
        },
      },
      {
        name: 'selectTempoMedio',
        component: 'ZdTextInput',
        cssClass: 'zd-px-2',
        label: 'Duração (minutos)',
        placeholder: '00:00',
        mask: '##:##',
        grid: {
          cols: 6,
        },
        validations: {
          required: {
            message: 'Campo Obrigatorio',
          },
          minLength: {
            limit: 5,
            message: 'Campo Obrigatorio',
          },
          maxLength: {
            limit: 5,
            message: 'Campo Obrigatorio',
          },
        },
      },
      {
        name: `selectFuncionario${n}`,
        component: 'ZdSelect',
        autocomplete: false,
        label: 'Funcionário',
        dataText: 'name',
        dataValue: 'value',
        validations: {
          required: {},
        },
        events: {
          onBeforeMount: '{{CadastroController.getDadosInfo}}',
          change: '{{CadastroController.validaCampoFuncionario}}',
        },
        datasource: {
          loadAll: true,
          data: [],
        },
        grid: {
          cols: 12,
        },
      },
    ];
  }

  public novoHorario() {
    Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = true;
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');
    const row = Metadata.getInstance<Row>('FormComFuncionariosDaTarefa');
    form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(0)[0] as IComponentRender);
    form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(0)[1] as IComponentRender);
    row.children.push(this.criaNovoHorario(row.children.length)[2] as IComponentRender);
  }

  public removeHorario() {
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');
    const row = Metadata.getInstance<Row>('FormComFuncionariosDaTarefa');
    if (row.children.length > 1) {
      row.children.splice(row.children.length - 1, 1);
      form.children.splice(form.children.length - 3, 2);
      if (row.children.length === 1) {
        Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = false;
        Metadata.getInstance<Switch>('switchFunionario').disabled = false;
      }
    }
  }

  public dataFormat(data: Date) {
        const dia  = data.getDate().toString();
        const diaF = (dia.length == 1) ? '0'+dia : dia;
        const mes  = (data.getMonth()+1).toString(); //+1 pois no getMonth Janeiro começa com zero.
        const mesF = (mes.length == 1) ? '0'+mes : mes;
        const anoF = data.getFullYear();
    return anoF+"/"+mesF+"/"+diaF;
  }

  public msg(text: string,tipo: string) {
    AlertService.show({
        name: 'basic-alert',
        text: text,
        top: true,
        left: false,
        timeout: 3000,
        color: tipo,
      });
  }

  public validaCampoFuncionario() {
    const dadosForm = Metadata.getInstance<Form>('FormComFuncionariosDaTarefa').value
    const validacao: IDictionary = []
    for (var property in dadosForm){
      if (dadosForm[property] != null) {
        (validacao.includes(dadosForm[property])) ? '' : validacao.push(dadosForm[property]);
      }
    }
    const swt = Metadata.getInstance<Switch>('switchFunionario');
    if (validacao.length > 1) {
      swt.value = false;
      swt.disabled = true;
    } else {
      swt.disabled = false;
    }
  }
  // Diego Sabino - Adicionado 5/4/21 - Sprint Alteração de tarefa sem funcionário alocado
  public mensagem (texto:  string, datas: IDictionary) {
  	DialogService.show({
  	  name: 'basic-dialog',
  	  type: 'warning',
  	  title: 'Atenção!',
  	  text: `${texto}`,
  	  buttons: [
      {
  		  name: 'cancel',
  		  component: 'ZdButton',
  		  outline: true,
  		  label: 'Voltar',
        events: { 
  		   click: () => DialogService.hide(),
  		  }
  		},
  		{
  		  name: 'yes',
  		  component: 'ZdButton',
  		  label: 'Criar',
  		  events: {
  		   click: () => this.postTarefa(datas),
  		  },
  		},
  	  ],
  	});
  }
  // Diego Sabino - Adicionado 5/4/21 - Sprint Alteração de tarefa sem funcionário alocado
  public postTarefa(datas: IDictionary) {
    DialogService.hide();
    this.carregandoModal.openPersistentModal();
    Http.post('/insertNovaTarefa', {
    	NMTAREFA: Metadata.getInstance<TextInput>('setNomeTarefa').value,
    	TIPOLOCAL: Metadata.getInstance<Select>('selectTipoLocal').value,
    	LOCAL: Metadata.getInstance<Select>('selectNomeLocal').value,
    	CHECKLIST: Metadata.getInstance<Select>('selectChecklist').value,
    	ITENSCHECK: Metadata.getInstance<Select>('selectTipoTarefa').value,
    	TAREFA: Metadata.getInstance<Select>('selectTarefa').value,
    	CRITERIOS: Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios').value,
    	DATA: datas,
    	HORAINICIO: Metadata.getInstance<Time>('selectHoraInicio').value,
    	DURACAO: Metadata.getInstance<TextInput>('selectTempoMedio').value,
    	NRFUNCIONARIO: Metadata.getInstance<Select>('selectFuncionario').value,
    }).then((response: IDictionary) => {
      const resposta = response.data.dataset.resposta.retorno;
      const erro = resposta.split(':');
      const texto = `<p>Operação Bloqueada!</p><p> Ocorreram erros durante a inserção da tarefa:</p><div class= "erro">${erro[10]}:${erro[11]}</div>`;
    	if (resposta != 'OK'){
        this.carregandoModal.closePersistentModal();
        DialogService.show({
      	  name: 'basic-dialog',
      	  type: 'warning',
      	  title: 'Atenção!',
      	  text: `${texto}`, 
      	  buttons: [
          {
      		  name: 'cancel',
      		  component: 'ZdButton',
      		  outline: true,
      		  label: 'Voltar',
            events: { 
      		   click: () => DialogService.hide(),
      		  }
      		}
      	  ],
      	});
       } else {
         this.carregandoModal.closePersistentModal();
         Router.replace('/Tarefas');
       }
    });
  }
}
